@@ -19,7 +19,8 @@ Cor2DataFrame <- function(x, n, v.na.replace=TRUE, row.names.unique=FALSE,
     ## Use unique row names if the row names are duplicated.
     if (row.names.unique) rownames(data) <- make.names(names(x), unique=TRUE)    
 
-    list(data=data, n=n, ylabels=dimnames(my.df)[[2]], vlabels=dimnames(acovR)[[2]])
+    list(data=data, n=n, obslabels=colnames(x[[1]]),
+         ylabels=dimnames(my.df)[[2]], vlabels=dimnames(acovR)[[2]])
 }
 
 
@@ -387,27 +388,48 @@ create.V <- function(x, type=c("Symm", "Diag", "Full"), as.mxMatrix=TRUE) {
 }
     
 osmasem <- function(model.name="osmasem", Mmatrix, Tmatrix, data,
-                    intervals.type = c("z", "LB"), mxModel.Args=NULL,
-                    mxRun.Args=NULL, suppressWarnings=TRUE,
-                    silent=TRUE, run=TRUE, ...) {
+                    subset=NULL, intervals.type = c("z", "LB"),
+                    mxModel.Args=NULL, mxRun.Args=NULL,
+                    suppressWarnings=TRUE, silent=TRUE, run=TRUE, ...) {
 
     intervals.type <- match.arg(intervals.type)
     switch(intervals.type,
            z = intervals <- FALSE,
            LB = intervals <- TRUE)
-    
+
+    ## Operator for not in
+    "%ni%" <- Negate("%in%")
+
+    ## Filter out variables not used in the analysis
+    if (!is.null(subset)) {
+        index  <- subset %ni% data$obslabels
+        if (any(index)) {
+            stop(paste0(paste0(subset[index], collapse = ", "), " are not in the ylabels.\n"))
+        }
+
+        temp <- .genCorNames(subset)
+        obslabels <- subset
+        ylabels <- temp$ylabels
+        vlabels <- temp$vlabels        
+    } else {
+        obslabels <- data$obslabels
+        ylabels <- data$ylabels
+        vlabels <- data$vlabels
+    }
+
     ## Create known sampling variance covariance matrix
-    Vmatrix <- create.V(data$vlabels, type="Symm", as.mxMatrix=TRUE)
-    
+    Vmatrix <- create.V(vlabels, type="Symm", as.mxMatrix=TRUE)
+
+    ## Dirty trick to avoid the warning of "no visible binding for global variable"
     mx.model <- eval(parse(text="mxModel(model=model.name,
                            mxData(observed=data$data, type='raw'),
                            mxExpectationNormal(covariance='expCov',
                                                means='vechsR',
-                                               dimnames=data$ylabels),
+                                               dimnames=ylabels),
                            mxFitFunctionML(), Mmatrix, Tmatrix, Vmatrix,
                            mxAlgebra(Tau2+V, name='expCov'),
                            mxCI(c('Amatrix', 'Smatrix', 'Tau2')))"))
-
+    
     ## Add additional arguments to mxModel
     if (!is.null(mxModel.Args)) {
         for (i in seq_along(mxModel.Args)) {
@@ -431,8 +453,11 @@ osmasem <- function(model.name="osmasem", Mmatrix, Tmatrix, data,
     }
 
     out <- list(call=match.call(), Mmatrix=Mmatrix, Tmatrix=Tmatrix,
-                Vmatrix=Vmatrix, data=data, mxModel.Args=mxModel.Args,
-                mxRun.Args=mxRun.Args, mx.model=mx.model, mx.fit=mx.fit)
+                Vmatrix=Vmatrix, data=data,
+                labels=list(obslabels=obslabels, ylabels=ylabels,
+                            vlabels=vlabels),
+                mxModel.Args=mxModel.Args, mxRun.Args=mxRun.Args,
+                mx.model=mx.model, mx.fit=mx.fit)
     class(out) <- 'osmasem'
     out
 }
@@ -453,20 +478,33 @@ coef.osmasem <- function(object, select=c("fixed", "all", "random"), ...) {
     mx.coef    
 }
 
-vcov.osmasem <- function(object, select=c("fixed", "all", "random"), ...) {
+vcov.osmasem <- function(object, select=c("fixed", "all", "random"),
+                         robust=FALSE, ...) {
     if (!is.element("osmasem", class(object)))
         stop("\"object\" must be an object of class \"osmasem\".")
-    
-    mx.vcov <- vcov(object$mx.fit)
-    ## index for untransformed random effects (not the correct ones!) 
-    index <- grep("Tau1_|Cor_", rownames(mx.vcov))
 
+    # labels of the parameters    
+    ## my.name <- summary(object$mx.fit)$parameters$name
+    my.name <- names(omxGetParameters(object$mx.fit))
+    my.name <- my.name[!is.na(my.name)]
+
+    ## index for untransformed random effects (not the correct ones!) 
+    index.random <- grep("Tau1_|Cor_", my.name) 
+    
     select <- match.arg(select)
-    switch(select,
-           fixed =  mx.vcov <- mx.vcov[-index, -index],
-           random = mx.vcov <- mx.vcov[index, index])
+    switch( select,
+         ## all = my.name <- my.name,
+         fixed =  my.name <- my.name[-index.random],
+         random = my.name <- my.name[index.random]
+         )
+
+    if (robust) {
+        out <- suppressMessages(imxRobustSE(object$mx.fit, details=TRUE))$cov
+    } else {
+        out <- vcov(object$mx.fit)
+    }
     if (select!="fixed") warning("\"Tau1_xx\" is not the variance component of the random effects.")
-    mx.vcov 
+    out[my.name, my.name]
 }
 
 VarCorr <- function(x, ...) {
@@ -501,8 +539,8 @@ VarCorr <- function(x, ...) {
 
 ## Fit a saturated model with either a diagonal or symmetric variance component of random effects
 .osmasemSatIndMod <- function(osmasem.obj=NULL, model=c("Saturated", "Independence"),
-                              Std.Error=FALSE, Tmatrix, data, mxModel.Args=NULL,
-                              mxRun.Args=NULL, suppressWarnings=TRUE,
+                              Std.Error=FALSE, Tmatrix, data, subset=NULL,
+                              mxModel.Args=NULL, mxRun.Args=NULL, suppressWarnings=TRUE,
                               silent=FALSE, run=TRUE, ...) {
 
     if (!is.null(osmasem.obj)) {
@@ -515,9 +553,9 @@ VarCorr <- function(x, ...) {
         p <- nrow(Tmatrix$Cor$values)
         data <- osmasem.obj$data
     } else {
-        p <- length(data$ylabels)
+        p <- length(osmasem.obj$labels$ylabels)
         ## Create known sampling variance covariance matrix
-        Vmatrix <- create.V(data$vlabels, type="Symm", as.mxMatrix=TRUE)
+        Vmatrix <- create.V(osmasem.obj$labels$vlabels, type="Symm", as.mxMatrix=TRUE)
     }    
 
     model <- match.arg(model)
@@ -531,15 +569,16 @@ VarCorr <- function(x, ...) {
         Mmatrix <- mxMatrix(type="Full", free=FALSE, labels=paste0("Mu", seq_len(p)),
                             nrow=1, ncol=p, name="Mu")        
     }
-    
+
+    ## Dirty trick to avoid the warning of "no visible binding for global variable"
     mx.model <- eval(parse(text="mxModel(model=model,
                                  mxData(observed=data$data, type='raw'),
                                  mxExpectationNormal(covariance='expCov',
                                                      means='Mu',
-                                                     dimnames=data$ylabels),
+                                                     dimnames=osmasem.obj$labels$ylabels),
                                  mxFitFunctionML(), Mmatrix, Tmatrix, Vmatrix,
                                  mxAlgebra(Tau2+V, name='expCov'))"))
-
+    
     ## Add additional arguments to mxModel
     if (!is.null(mxModel.Args)) {
         for (i in seq_along(mxModel.Args)) {
@@ -575,7 +614,8 @@ anova.osmasem <- function(object, ..., all=FALSE) {
   mxCompare(base=base, comparison=comparison, all=all)
 }
 
-summary.osmasem <- function(object, Saturated=FALSE, numObs, ...) {
+summary.osmasem <- function(object, Saturated=FALSE, numObs,
+                            robust=FALSE, ...) {
     if (!is.element("osmasem", class(object)))
         stop("\"object\" must be an object of class \"osmasem\".")
 
@@ -600,6 +640,12 @@ summary.osmasem <- function(object, Saturated=FALSE, numObs, ...) {
         out <- summary(object$mx.fit, numObs=numObs, ...)
     }
 
+    ## Modified the SE with robust SE
+    if (robust) {
+        robust.SE <- suppressMessages(imxRobustSE(object$mx.fit))
+        out$parameters[, "Std.Error"] <- robust.SE[out$parameters$name]
+    }
+
     ## Additional output to the mx summary
     out$parameters$`z value` <- with(out$parameters, Estimate/Std.Error)
     out$parameters$`Pr(>|z|)` <- 2*(1-pnorm(abs(out$parameters$`z value`)))
@@ -643,6 +689,21 @@ osmasemSRMR <- function(x) {
     sqrt(mean(vechs(sampleR-impliedR)^2))
 }
 
+## Generate names for OSMASEM
+## x: a vector of observed variables
+## output: a vector of correlation coefficients and sampling covariance matrix
+## This function is used to select data in data created by Cor2DataFrame()
+.genCorNames <- function(x) {
+  ## Names for the correlation elements
+  psNames <- vechs(outer(x, x, paste, sep = "_"))
+  
+  ## Names for the sampling covariance matrix of the correlation vector
+  psCovNames <- paste("C(", outer(psNames, psNames, paste, sep = " "), ")", sep="")
+  psCovNames <- vech(matrix(psCovNames, nrow=length(psNames), ncol=length(psNames)))
+
+  list(ylabels=psNames, vlabels=psCovNames)
+}  
+
 ## RAMmodelV <- function(Amatrix, Smatrix, Fmatrix, Mmatrix, Vmatrix,
 ##                       data, obs.var, ...) {
 
